from unittest import TestCase, mock
from unittest.mock import ANY

from jovian._version import __version__
from jovian.tests.resources import fake_creds
from jovian.utils.api import (_h, create_gist_simple, get_current_user, get_gist, get_gist_access, post_block,
                              post_blocks, post_records, post_slack_message, upload_file)
from jovian.utils.credentials import write_creds
from jovian.utils.error import ApiError


class MockResponse:
    def __init__(self, json_data, status_code, text=""):
        self.json_data = json_data
        self.status_code = status_code
        self.text = text

    def json(self):
        return self.json_data


def mock_requests_post(url, *args, **kwargs):
    if url == 'https://api-staging.jovian.ai/gist/create' or \
       url == 'https://api-staging.jovian.ai/gist/fake_gist_slug/upload':
        if kwargs['headers']['Authorization'] == 'Bearer fake_api_key':
            data = {
                'data': {
                    'message': 'Gist created successfully'
                }
            }
            return MockResponse(data, status_code=200)
        else:
            data = {
                "errors": [
                    {
                        "code": 404,
                        "message": "Gist not found"
                    }
                ],
                "success": False
            }
            return MockResponse(data, status_code=404)
    elif url == 'https://api-staging.jovian.ai/data/record':
        if kwargs['headers']['Authorization'] == 'Bearer fake_api_key':
            data = {
                'data': {
                    'message': 'Data logged successfully'
                }
            }
            return MockResponse(data, status_code=200)
        else:
            data = {
                "errors": [
                    {
                        "code": 500,
                        "message": "Internal Server Error"
                    }
                ],
                "success": False
            }
            return MockResponse(data, status_code=500)

    elif url == 'https://api-staging.jovian.ai/data/fake_gist_slug/commit':
        if kwargs['headers']['Authorization'] == 'Bearer fake_api_key':
            data = {
                'data': {
                    'message': 'Data logged successfully'
                }
            }
            return MockResponse(data, status_code=200)
        else:
            data = {
                "errors": [
                    {
                        "code": 404,
                        "message": "Gist not found"
                    }
                ],
                "success": False
            }
            return MockResponse(data, status_code=404)

    elif url == 'https://api-staging.jovian.ai/slack/notify':
        if kwargs['headers']['Authorization'] == 'Bearer fake_api_key':
            data = {
                "data": {
                    "messageId": "60023f08f3b54801bd58cf6b37067ed6",
                    "messageSent": True
                },
                "success": True
            }
            return MockResponse(data, 200)
        elif kwargs['headers']['Authorization'] == 'Bearer fake_invalid_api_key':
            data = {
                "errors": [
                    {
                        "code": 401,
                        "message": "The token is invalid"
                    }
                ],
                "success": False
            }

            return MockResponse(data, 401)

        elif kwargs['headers']['Authorization'] == 'Bearer fake_expired_api_key':
            data = {
                "errors": [
                    {
                        "code": 200,
                        "message": "The token has expired"
                    }
                ],
                "success": False
            }

            return MockResponse(data, 200)


def mock_requests_get(url, *args, **kwargs):
    if url == 'https://api-staging.jovian.ai/user/profile':
        if kwargs['headers']['Authorization'] == 'Bearer fake_api_key':
            data = {
                "data": {
                    "id": 47,
                    "username": "rohit"
                },
                "success": True
            }

            return MockResponse(data, status_code=200)
        else:
            data = {
                "errors": [
                    {
                        "code": 401,
                        "message": "Signature verification failed"
                    }
                ],
                "success": False
            }
            return MockResponse(data, status_code=401)

    elif url == 'https://api-staging.jovian.ai/user/rohit/gist/demo-notebook' or \
            url == 'https://api-staging.jovian.ai/user/rohit/gist/demo-notebook?gist_version=3' or \
            url == 'https://api-staging.jovian.ai/gist/f67108fc906341d8b15209ce88ebc3d2':

        data = {
            "data": {
                "archived": False,
                "clonesCount": 0,
                "createdAt": 1578718557994,
                "currentUser": {
                    "id": 47,
                    "username": "rohit"
                },
                "deleted": False,
                "description": None
            }
        }

        return MockResponse(data, status_code=200)

    # Dictionary containing mock responses
    response_dict = {
        'https://api-staging.jovian.ai/gist/fake_nonexistent_gist': MockResponse({
            "errors": [
                {
                    "code": 404,
                    "message": "Gist not found"
                }
            ],
            "success": False
        }, status_code=404),

        'https://api-staging.jovian.ai/gist/fake_gist_too_large': MockResponse({
            "message": "Internal Server Error"
        }, status_code=500),

        'https://api-staging.jovian.ai/gist/f67108fc906341d8b15209ce88ebc3d2/check-access': MockResponse({
            "data": {
                "currentUser": {
                    "id": 47,
                    "username": "rohit"
                },
                "read": True,
                "slug": "f67108fc906341d8b15209ce88ebc3d2",
                "write": True
            },
            "success": True
        }, status_code=200),

        'https://api-staging.jovian.ai/gist/fake_nonexistent_gist/check-access': MockResponse({
            "errors": [
                {
                    "code": 404,
                    "message": "Gist not found"
                }
            ],
            "success": False
        }, status_code=404)
    }

    return response_dict[url]


def test_h():
    with fake_creds('.jovian', 'credentials.json'):
        expected_result = {"Authorization": "Bearer fake_api_key",
                           "x-jovian-source": "library",
                           "x-jovian-library-version": __version__,
                           "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                           "x-jovian-org": "staging"}

        assert _h() == expected_result


class TestGetCurrentUser(TestCase):

    @mock.patch("requests.get", side_effect=mock_requests_get)
    def test_get_current_user(self, mock_requests_get):
        with fake_creds('.jovian', 'credentials.json'):
            get_current_user()

            mock_requests_get.assert_called_with(
                'https://api-staging.jovian.ai/user/profile',
                headers={"Authorization": "Bearer fake_api_key",
                         "x-jovian-source": "library",
                         "x-jovian-library-version": __version__,
                         "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                         "x-jovian-org": "staging"},
                params=None)

    @mock.patch("jovian.utils.request.get_api_key", return_value="fake_invalid_api_key")
    @mock.patch("requests.get", side_effect=mock_requests_get)
    def test_get_current_user_raises_exception(self, mock_requests_get, mock_request_get_api_key):
        with fake_creds('.jovian-invalid-key', 'credentials.json'):
            # setUp
            creds = {"WEBAPP_URL": "https://staging.jovian.ml/",
                     "GUEST_KEY": "b6538d4dfde04fcf993463a828a9cec6",
                     "API_URL": "https://api-staging.jovian.ai",
                     "API_KEY": "fake_invalid_api_key",
                     "ORG_ID": "staging"}
            write_creds(creds)

            with self.assertRaises(Exception) as context:
                get_current_user()

            mock_requests_get.assert_called_with(
                'https://api-staging.jovian.ai/user/profile',
                headers={"Authorization": "Bearer fake_invalid_api_key",
                         "x-jovian-source": "library",
                         "x-jovian-library-version": __version__,
                         "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                         "x-jovian-org": "staging"},
                params=None)

            assert context.exception.args[0] == 'Failed to fetch current user profile. (HTTP 401) Signature verification failed'


class TestGetGist(TestCase):

    @mock.patch("jovian.utils.request.get_api_key", return_value="fake_api_key")
    @mock.patch("requests.get", side_effect=mock_requests_get)
    def test_get_gist(self, mock_requests_get, mock_request_get_api_key):
        with fake_creds('.jovian', 'credentials.json'):
            get_gist('rohit/demo-notebook')
            mock_requests_get.assert_called_with(
                'https://api-staging.jovian.ai/user/rohit/gist/demo-notebook',
                headers={"Authorization": "Bearer fake_api_key", "x-jovian-source": "library",
                         "x-jovian-library-version": __version__, "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                         "x-jovian-org": "staging"},
                params=None)

            get_gist('rohit/demo-notebook', version=3)
            mock_requests_get.assert_called_with(
                'https://api-staging.jovian.ai/user/rohit/gist/demo-notebook?gist_version=3',
                headers={"Authorization": "Bearer fake_api_key",
                         "x-jovian-source": "library",
                         "x-jovian-library-version": __version__,
                         "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                         "x-jovian-org": "staging"},
                params=None)

            get_gist('f67108fc906341d8b15209ce88ebc3d2')
            mock_requests_get.assert_called_with(
                'https://api-staging.jovian.ai/gist/f67108fc906341d8b15209ce88ebc3d2',
                headers={"Authorization": "Bearer fake_api_key", "x-jovian-source": "library",
                         "x-jovian-library-version": __version__, "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                         "x-jovian-org": "staging"},
                params=None)

    @mock.patch("jovian.utils.request.get_api_key", return_value="fake_api_key")
    @mock.patch("requests.get", side_effect=mock_requests_get)
    def test_get_gist_not_found(self, mock_requests_get, mock_get_api_key):
        with fake_creds('.jovian', 'credentials.json'):
            assert get_gist('fake_nonexistent_gist') == False

    @mock.patch("jovian.utils.request.get_api_key", return_value="fake_api_key")
    @mock.patch("requests.get", side_effect=mock_requests_get)
    def test_get_gist_raises_exception(self, mock_requests_get, mock_get_api_key):
        with fake_creds('.jovian', 'credentials.json'):
            with self.assertRaises(Exception) as context:
                get_gist('fake_gist_too_large')

            assert context.exception.args[0] == 'Failed to retrieve metadata for notebook "fake_gist_too_large":' + \
                                                ' (HTTP 500) Internal Server Error'


class TestGetGistAccess(TestCase):

    @mock.patch("jovian.utils.request.get_api_key", return_value="fake_api_key")
    @mock.patch("requests.get", side_effect=mock_requests_get)
    def test_get_gist_access(self, mock_requests_get, mock_get_api_key):
        with fake_creds('.jovian', 'credentials.json'):
            get_gist_access('f67108fc906341d8b15209ce88ebc3d2')
            mock_requests_get.assert_called_with(
                'https://api-staging.jovian.ai/gist/f67108fc906341d8b15209ce88ebc3d2/check-access',
                headers={"Authorization": "Bearer fake_api_key",
                         "x-jovian-source": "library",
                         "x-jovian-library-version": __version__,
                         "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                         "x-jovian-org": "staging"},
                params=None)

    @mock.patch("jovian.utils.request.get_api_key", return_value="fake_api_key")
    @mock.patch("requests.get", side_effect=mock_requests_get)
    def test_get_gist_access_raises_exception(self, mock_requests_get, mock_get_api_key):
        with fake_creds('.jovian', 'credentials.json'):
            with self.assertRaises(Exception) as context:
                get_gist_access('fake_nonexistent_gist')

            mock_requests_get.assert_called_with(
                'https://api-staging.jovian.ai/gist/fake_nonexistent_gist/check-access',
                headers={"Authorization": "Bearer fake_api_key",
                         "x-jovian-source": "library",
                         "x-jovian-library-version": __version__,
                         "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                         "x-jovian-org": "staging"},
                params=None)

            assert context.exception.args[0] == 'Failed to retrieve access permission for notebook' + \
                ' "fake_nonexistent_gist" (retry with new_project=True to create a new notebook):' +\
                ' (HTTP 404) Gist not found'


class TestCreateGistSimple(TestCase):

    @mock.patch("requests.post", side_effect=mock_requests_post)
    def test_create_gist_simple_no_gist_slug(self, mock_requests_post):
        with fake_creds('.jovian', 'credentials.json'):
            create_gist_simple(filename='jovian/tests/resources/creds/.jovian/credentials.json',
                               title='Credentials',
                               privacy='private',
                               version_title='first version')

            mock_requests_post.assert_called_with(
                'https://api-staging.jovian.ai/gist/create',
                data={'visibility': 'private', 'public': False, 'title': 'Credentials',
                      'version_title': 'first version'},
                files={'files': ('jovian/tests/resources/creds/.jovian/credentials.json', ANY)},
                headers={"Authorization": "Bearer fake_api_key",
                         "x-jovian-source": "library",
                         "x-jovian-library-version": __version__,
                         "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                         "x-jovian-org": "staging"},
                json=None)

    @mock.patch("requests.post", side_effect=mock_requests_post)
    def test_create_gist_simple_with_gist_slug(self, mock_requests_post):
        with fake_creds('.jovian', 'credentials.json'):
            create_gist_simple(filename='jovian/tests/resources/creds/.jovian/credentials.json',
                               gist_slug='fake_gist_slug',
                               title='Credentials',
                               version_title='first version')

            mock_requests_post.assert_called_with(
                'https://api-staging.jovian.ai/gist/fake_gist_slug/upload',
                data={'version_title': 'first version'},
                files={'files': ('jovian/tests/resources/creds/.jovian/credentials.json', ANY)},
                headers={"Authorization": "Bearer fake_api_key",
                         "x-jovian-source": "library",
                         "x-jovian-library-version": __version__,
                         "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                         "x-jovian-org": "staging"},
                json=None)

    @mock.patch("requests.post", side_effect=mock_requests_post)
    def test_create_gist_simple_raises_api_error(self, mock_requests_post):
        with fake_creds('.jovian', 'credentials.json'):
            # setUp
            creds = {"WEBAPP_URL": "https://staging.jovian.ml/",
                     "GUEST_KEY": "b6538d4dfde04fcf993463a828a9cec6",
                     "API_URL": "https://api-staging.jovian.ai",
                     "API_KEY": "fake_invalid_api_key",
                     "ORG_ID": "staging"}
            write_creds(creds)

            with self.assertRaises(ApiError) as context:
                create_gist_simple(filename='jovian/tests/resources/creds/.jovian/credentials.json',
                                   title='Credentials',
                                   version_title='first version')

            mock_requests_post.assert_called_with(
                'https://api-staging.jovian.ai/gist/create',
                data={'visibility': 'auto', 'public': True, 'title': 'Credentials', 'version_title': 'first version'},
                files={'files': ('jovian/tests/resources/creds/.jovian/credentials.json', ANY)},
                headers={"Authorization": "Bearer fake_invalid_api_key",
                         "x-jovian-source": "library",
                         "x-jovian-library-version": __version__,
                         "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                         "x-jovian-org": "staging"},
                json=None)

            assert context.exception.args[0] == 'File upload failed: (HTTP 404) Gist not found'


class TestUploadFile(TestCase):

    @mock.patch("requests.post", side_effect=mock_requests_post)
    def test_upload_file(self, mock_requests_post):
        with fake_creds('.jovian', 'credentials.json'):
            with open('jovian/tests/resources/creds/.jovian/credentials.json', 'rb') as f:
                upload_file(gist_slug='fake_gist_slug',
                            file=('credentials.json', f),
                            folder='.jovian',
                            artifact=True,
                            version_title='fake_version_title')

                mock_requests_post.assert_called_with(
                    'https://api-staging.jovian.ai/gist/fake_gist_slug/upload',
                    data={'artifact': 'true', 'folder': '.jovian', 'version_title': 'fake_version_title'},
                    files={'files': ('credentials.json', ANY)},
                    headers={"Authorization": "Bearer fake_api_key",
                             "x-jovian-source": "library",
                             "x-jovian-library-version": __version__,
                             "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                             "x-jovian-org": "staging"},
                    json=None)

    @mock.patch("jovian.utils.request.get_api_key", return_value="fake_invalid_api_key")
    @mock.patch("requests.post", side_effect=mock_requests_post)
    def test_upload_file_raises_api_error(self, mock_requests_post, mock_get_api_key):
        with fake_creds('.jovian', 'credentials.json'):
            # setUp
            creds = {"WEBAPP_URL": "https://staging.jovian.ml/",
                     "GUEST_KEY": "b6538d4dfde04fcf993463a828a9cec6",
                     "API_URL": "https://api-staging.jovian.ai",
                     "API_KEY": "fake_invalid_api_key",
                     "ORG_ID": "staging"}
            write_creds(creds)

            with self.assertRaises(ApiError) as context:
                with open('jovian/tests/resources/creds/.jovian/credentials.json', 'rb') as f:
                    upload_file(gist_slug='fake_gist_slug',
                                file=('credentials.json', f),
                                folder='.jovian',
                                artifact=True,
                                version_title='fake_version_title')

            mock_requests_post.assert_called_with(
                'https://api-staging.jovian.ai/gist/fake_gist_slug/upload',
                data={'artifact': 'true', 'folder': '.jovian', 'version_title': 'fake_version_title'},
                files={'files': ('credentials.json', ANY)},
                headers={"Authorization": "Bearer fake_invalid_api_key",
                         "x-jovian-source": "library",
                         "x-jovian-library-version": __version__,
                         "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                         "x-jovian-org": "staging"},
                json=None)

            assert context.exception.args[0] == 'File upload failed: (HTTP 404) Gist not found'


class TestPostBlocks(TestCase):

    @mock.patch("requests.post", side_effect=mock_requests_post)
    def test_post_blocks(self, mock_requests_post):

        with fake_creds('.jovian', 'credentials.json'):
            blocks = [{'data': {'key': 'value'}, 'record_type': 'metrics'}]
            post_blocks(blocks)

            mock_requests_post.assert_called_with(
                'https://api-staging.jovian.ai/data/record',
                data=None,
                headers={"Authorization": "Bearer fake_api_key",
                         "x-jovian-source": "library",
                         "x-jovian-library-version": __version__,
                         "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                         "x-jovian-org": "staging"},
                json=[{'data': {'key': 'value'}, 'record_type': 'metrics'}])

    @mock.patch("jovian.utils.request.get_api_key", return_value="fake_invalid_api_key")
    @mock.patch("requests.post", side_effect=mock_requests_post)
    def test_post_blocks_raises_api_error(self, mock_requests_post, mock_get_api_key):
        with fake_creds('.jovian-invalid-key', 'credentials.json'):
            # setUp
            creds = {"WEBAPP_URL": "https://staging.jovian.ml/",
                     "GUEST_KEY": "b6538d4dfde04fcf993463a828a9cec6",
                     "API_URL": "https://api-staging.jovian.ai",
                     "API_KEY": "fake_invalid_api_key",
                     "ORG_ID": "staging"}
            write_creds(creds)

            blocks = [{'data': {'key': 'value'}, 'record_type': 'metrics'}]

            with self.assertRaises(ApiError) as context:
                post_blocks(blocks)

            mock_requests_post.assert_called_with(
                'https://api-staging.jovian.ai/data/record',
                data=None,
                headers={"Authorization": "Bearer fake_invalid_api_key",
                         "x-jovian-source": "library",
                         "x-jovian-library-version": __version__,
                         "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                         "x-jovian-org": "staging"},
                json=[{'data': {'key': 'value'}, 'record_type': 'metrics'}])

            assert context.exception.args[0] == 'Data logging failed: (HTTP 500) Internal Server Error'


class TestPostBlock(TestCase):
    @mock.patch("jovian.utils.api.timestamp_ms", return_value=1582550133094)
    @mock.patch("requests.post", side_effect=mock_requests_post)
    def test_post_block(self, mock_requests_post, mock_timestamp):
        with fake_creds('.jovian', 'credentials.json'):
            post_block('metrics', {'key': 'value'})

            mock_requests_post.assert_called_with(
                'https://api-staging.jovian.ai/data/record',
                data=None,
                headers={"Authorization": "Bearer fake_api_key",
                         "x-jovian-source": "library",
                         "x-jovian-library-version": __version__,
                         "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                         "x-jovian-org": "staging"},
                json=[{'localTimestamp': 1582550133094, 'data': 'metrics', 'recordType': {'key': 'value'}}])

    @mock.patch("jovian.utils.api.timestamp_ms", return_value=1582550133094)
    @mock.patch("jovian.utils.request.get_api_key", return_value="fake_invalid_api_key")
    @mock.patch("requests.post", side_effect=mock_requests_post)
    def test_post_block_raises_api_error(self, mock_requests_post, mock_get_api_key, mock_timestamp):
        with fake_creds('.jovian-invalid-key', 'credentials.json'):
            # setUp
            creds = {"WEBAPP_URL": "https://staging.jovian.ml/",
                     "GUEST_KEY": "b6538d4dfde04fcf993463a828a9cec6",
                     "API_URL": "https://api-staging.jovian.ai",
                     "API_KEY": "fake_invalid_api_key",
                     "ORG_ID": "staging"}
            write_creds(creds)

            with self.assertRaises(ApiError) as context:
                post_block('metrics', {'key': 'value'})

            mock_requests_post.assert_called_with(
                'https://api-staging.jovian.ai/data/record',
                data=None,
                headers={"Authorization": "Bearer fake_invalid_api_key",
                         "x-jovian-source": "library",
                         "x-jovian-library-version": __version__,
                         "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                         "x-jovian-org": "staging"},
                json=[{'localTimestamp': 1582550133094, 'data': 'metrics', 'recordType': {'key': 'value'}}])

            assert context.exception.args[0] == 'Data logging failed: (HTTP 500) Internal Server Error'


class TestPostRecords(TestCase):
    @mock.patch("requests.post", side_effect=mock_requests_post)
    def test_post_records(self, mock_requests_post):
        with fake_creds('.jovian', 'credentials.json'):
            post_records('fake_gist_slug', {'key': 'value'})
            mock_requests_post.assert_called_with(
                'https://api-staging.jovian.ai/data/fake_gist_slug/commit',
                data=None,
                headers={"Authorization": "Bearer fake_api_key",
                         "x-jovian-source": "library",
                         "x-jovian-library-version": __version__,
                         "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                         "x-jovian-org": "staging"},
                json={'key': 'value'})

    @mock.patch("jovian.utils.request.get_api_key", return_value="fake_invalid_api_key")
    @mock.patch("requests.post", side_effect=mock_requests_post)
    def test_post_records_raises_api_error(self, mock_requests_post, mock_get_api_key):
        with fake_creds('.jovian-invalid-key', 'credentials.json'):
            # setUp
            creds = {"WEBAPP_URL": "https://staging.jovian.ml/",
                     "GUEST_KEY": "b6538d4dfde04fcf993463a828a9cec6",
                     "API_URL": "https://api-staging.jovian.ai",
                     "API_KEY": "fake_invalid_api_key",
                     "ORG_ID": "staging"}
            write_creds(creds)

            with self.assertRaises(ApiError) as context:
                post_records('fake_gist_slug', {'key': 'value'})

            mock_requests_post.assert_called_with(
                'https://api-staging.jovian.ai/data/fake_gist_slug/commit',
                data=None,
                headers={"Authorization": "Bearer fake_invalid_api_key",
                         "x-jovian-source": "library",
                         "x-jovian-library-version": __version__,
                         "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                         "x-jovian-org": "staging"},
                json={'key': 'value'})

            assert context.exception.args[0] == 'Data logging failed: (HTTP 404) Gist not found'


class TestPostSlackMessage(TestCase):

    @mock.patch("requests.post", side_effect=mock_requests_post)
    def test_post_slack_message(self, mock_requests_post):
        with fake_creds('.jovian-notify', 'credentials.json'):
            # setUp
            creds = {"WEBAPP_URL": "https://staging.jovian.ml/",
                     "GUEST_KEY": "b6538d4dfde04fcf993463a828a9cec6",
                     "API_URL": "https://api-staging.jovian.ai",
                     "API_KEY": "fake_api_key",
                     "ORG_ID": "staging"}
            write_creds(creds)

            post_slack_message({'key': 'value'})

            mock_requests_post.assert_called_with(
                'https://api-staging.jovian.ai/slack/notify',
                data=None,
                headers={"Authorization": "Bearer fake_api_key",
                         "x-jovian-source": "library",
                         "x-jovian-library-version": __version__,
                         "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                         "x-jovian-org": "staging"},
                json={'key': 'value'})

    @mock.patch("jovian.utils.request.get_api_key", return_value="fake_invalid_api_key")
    @mock.patch("requests.post", side_effect=mock_requests_post)
    def test_post_slack_message_raises_api_error(
            self, mock_requests_post, mock_request_get_api_key):
        with fake_creds('.jovian-notify', 'credentials.json'):
            # setUp
            creds = {"WEBAPP_URL": "https://staging.jovian.ml/",
                     "GUEST_KEY": "b6538d4dfde04fcf993463a828a9cec6",
                     "API_URL": "https://api-staging.jovian.ai",
                     "API_KEY": "fake_invalid_api_key",
                     "ORG_ID": "staging"}
            write_creds(creds)

            data = {'key': 'value'}

            with self.assertRaises(ApiError):
                post_slack_message(data)

            mock_requests_post.assert_called_with(
                'https://api-staging.jovian.ai/slack/notify',
                data=None,
                headers={"Authorization": "Bearer fake_invalid_api_key",
                         "x-jovian-source": "library",
                         "x-jovian-library-version": __version__,
                         "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                         "x-jovian-org": "staging"},
                json={'key': 'value'})

    @mock.patch("jovian.utils.request.get_api_key", return_value="fake_invalid_api_key")
    @mock.patch("requests.post", side_effect=mock_requests_post)
    def test_post_slack_message_safe(self, mock_requests_post, mock_request_get_api_key):
        with fake_creds('.jovian-notify', 'credentials.json'):
            # setUp
            creds = {"WEBAPP_URL": "https://staging.jovian.ml/",
                     "GUEST_KEY": "b6538d4dfde04fcf993463a828a9cec6",
                     "API_URL": "https://api-staging.jovian.ai",
                     "API_KEY": "fake_invalid_api_key",
                     "ORG_ID": "staging"}
            write_creds(creds)

            data = {'key': 'value'}

            assert post_slack_message(data, safe=True) == {'data': {'messageSent': False}}

            mock_requests_post.assert_called_with(
                'https://api-staging.jovian.ai/slack/notify',
                data=None,
                headers={"Authorization": "Bearer fake_invalid_api_key",
                         "x-jovian-source": "library",
                         "x-jovian-library-version": __version__,
                         "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                         "x-jovian-org": "staging"},
                json={'key': 'value'})



import os
import shutil
from unittest import TestCase, mock

from jovian.utils.git import (commit, get_branch, get_current_commit, get_relative_path, get_remote,
                              get_repository_root, git_commit_push, git_push, is_git)

path = 'mock_git_repo'


class GitMasterBranch(TestCase):
    path = 'mock_git_repo'
    origin_url = 'https://github.com/rohitsanj/mock_repo.git'

    def setUp(self):
        os.mkdir(self.path)
        os.chdir(self.path)
        os.system("""git init
        mkdir -p ./nested/folder/deep && touch ./nested/folder/deep/sample.ipynb""")
        os.chdir('nested/folder/deep')
        os.system('git add . && git commit -m "initial commit"')

    def tearDown(self):
        os.chdir('../../../../')
        shutil.rmtree(self.path)


class GitSampleBranch(GitMasterBranch):

    def setUp(self):
        super().setUp()
        os.system('git checkout -b sample_branch')


class TempFolder(TestCase):
    sample_folder = '/tmp/sample_folder'
    owd = os.getcwd()

    def setUp(self):
        os.mkdir(self.sample_folder)
        os.chdir(self.sample_folder)

    def tearDown(self):
        shutil.rmtree(self.sample_folder)
        os.chdir(self.owd)


class TestIsGit(GitMasterBranch):

    def test_is_git(self):
        self.assertTrue(is_git())


class TestIsNotGit(TempFolder):

    def test_is_not_git(self):
        self.assertFalse(is_git())


class TestGetBranchMaster(GitMasterBranch):

    def test_get_branch(self):
        expected_result = "master"
        self.assertEqual(get_branch(), expected_result)


class TestGetBranchSample(GitSampleBranch):

    def test_get_branch(self):
        expected_result = "sample_branch"
        self.assertEqual(get_branch(), expected_result)


class TestGetRemote(GitMasterBranch):

    def test_get_remote(self):
        expected_result = ''
        self.assertEqual(get_remote(), expected_result)


class TestGetRemoteNotSet(GitMasterBranch):
    def setUp(self):
        super().setUp()
        os.system('git remote add origin https://github.com/rohitsanj/mock_repo.git')

    def test_get_remote(self):
        expected_result = 'https://github.com/rohitsanj/mock_repo'
        self.assertEqual(get_remote(), expected_result)


class TestGetRepositoryRoot(GitMasterBranch):
    def setUp(self):
        super().setUp()
        os.makedirs('folder1/folder2')
        os.system('cd folder1/folder2')

    def test_get_repository_root(self):
        expected_result = os.path.join(os.path.abspath('../../../'))
        self.assertEqual(get_repository_root(), expected_result)

    def tearDown(self):
        os.system('cd -')
        shutil.rmtree('folder1')
        super().tearDown()


class TestGetCurrentCommit(GitMasterBranch):

    def test_get_current_commit(self):
        expected_result = os.popen('git rev-parse HEAD').read().strip()

        self.assertEqual(get_current_commit(), expected_result)


class TestGetCurrentCommitSample(GitSampleBranch):

    def test_get_current_commit(self):
        expected_result = os.popen('git rev-parse HEAD').read().strip()

        self.assertEqual(get_current_commit(), expected_result)


class TestCommit(GitMasterBranch):
    def setUp(self):
        super().setUp()
        os.system('touch file')

    def test_commit(self):
        message = 'sample commit'
        expected_result = 'cd ' + get_repository_root() + ' && git add . &&  git commit -a -m "sample commit" && cd -'

        self.assertEqual(commit(message), 0)


class TestGitPush(GitMasterBranch):

    @mock.patch("os.system")
    def test_git_push(self, mock_system):
        expected_result = "git push origin master"

        git_push()
        mock_system.assert_called_with(expected_result)


class TestGitPushSample(GitSampleBranch):

    @mock.patch("os.system")
    def test_git_push_sample(self, mock_system):
        expected_result = "git push origin sample_branch"

        git_push()
        mock_system.assert_called_with(expected_result)


class TestGitCommitPush(GitMasterBranch):

    def test_git_commit_push(self):
        message = 'sample commit'
        expected_result = {
            'remote': get_remote(),
            'branch': get_branch(),
            'commit': get_current_commit()
        }

        self.assertEqual(git_commit_push(message), expected_result)


class TestGetRelativePath(GitMasterBranch):

    def test_get_relative_path(self):
        expected_result = "nested/folder/deep"

        self.assertEqual(get_relative_path(), expected_result)



import json
from unittest import TestCase, mock

from jovian.utils.request import _msg, get, post, pretty, retry


class TestMsg(TestCase):

    def test_msg_errors(self):
        mock_res = mock.Mock()
        mock_res.json.return_value = {
            'errors': [
                {
                    'message': 'this is a fake requests error',
                    'code': 400
                }
            ]
        }

        expected_result = 'this is a fake requests error'
        self.assertEqual(_msg(mock_res), expected_result)

    def test_msg_message(self):
        mock_res = mock.Mock()
        mock_res.json.return_value = {
            'message': 'this is a fake response message'
        }

        expected_result = 'this is a fake response message'
        self.assertEqual(_msg(mock_res), expected_result)

    def test_msg_msg(self):
        mock_res = mock.Mock()
        mock_res.json.return_value = {
            'msg': 'this is a fake response message'
        }

        expected_result = 'this is a fake response message'
        self.assertEqual(_msg(mock_res), expected_result)

    def test_msg_exception_text(self):
        mock_res = mock.Mock(return_value=ValueError('fake value error'))
        mock_res.text = 'this is a fake response text'

        expected_result = 'this is a fake response text'
        self.assertEqual(_msg(mock_res), expected_result)

    def test_msg_exception_no_text(self):
        mock_res = mock.Mock()
        mock_res.json.return_value = ValueError('fake value error')
        mock_res.text = ''
        expected_result = 'Something went wrong'
        self.assertEqual(_msg(mock_res), expected_result)


class TestPretty(TestCase):

    def test_pretty(self):
        mock_res = mock.Mock()
        mock_res.status_code = 200
        mock_res.json.return_value = {
            'msg': 'this is a fake response message'
        }

        expected_result = '(HTTP 200) this is a fake response message'
        self.assertEqual(pretty(mock_res), expected_result)


class MockResponse:
    def __init__(self, json_data, status_code):
        self.json_data = json_data
        self.status_code = status_code

    def json(self):
        return self.json_data


def mocked_requests(url, *args, **kwargs):

    if url == 'http://someurl.com/test.json':
        return MockResponse({'key': 'value'}, 200)
    elif url == 'http://someurl.com/noauth':
        return MockResponse({'message': 'invalid creds'}, 401)
    elif url == 'http://someurl.com/create':
        return MockResponse({'message': 'created successfully'}, 201)
    return MockResponse(None, 404)


class TestGet(TestCase):

    @mock.patch('requests.get', side_effect=mocked_requests)
    def test_get(self, mock_get):
        expected_result = {'key': 'value'}

        self.assertEqual(get('http://someurl.com/test.json', headers={'key': 'value'}).json(), expected_result)

    @mock.patch('requests.get', side_effect=mocked_requests)
    def test_get_called_once(self, mock_get):
        get('http://someurl.com/test.json', headers={'key': 'value'})

        assert mock_get.call_count == 1

    @mock.patch('jovian.utils.request.get_api_key', mock.Mock(return_value="fake_api_key"))
    @mock.patch('requests.get', side_effect=mocked_requests)
    def test_get_called_twice(self, mock_get):
        get('http://someurl.com/noauth', headers={'key': 'value'})

        assert mock_get.call_count == 2


class TestPost(TestCase):

    @mock.patch('requests.post', side_effect=mocked_requests)
    def test_get(self, mock_post):
        expected_result = {"message": "created successfully"}

        self.assertEqual(post('http://someurl.com/create', headers={'key': 'value'}).json(), expected_result)

    @mock.patch('requests.post', side_effect=mocked_requests)
    def test_post_called_once(self, mock_post):
        post('http://someurl.com/create', headers={'key': 'value'})

        assert mock_post.call_count == 1

    @mock.patch('jovian.utils.request.get_api_key', mock.Mock(return_value="fake_api_key"))
    @mock.patch('requests.post', side_effect=mocked_requests)
    def test_post_called_twice(self, mock_post):
        post('http://someurl.com/noauth', headers={'key': 'value'})

        assert mock_post.call_count == 2



from contextlib import contextmanager
from jovian.utils import credentials
from jovian.utils.credentials import write_creds, purge_config


@contextmanager
def fake_creds(config_dir, creds_filename):
    _d, _f = credentials.CONFIG_DIR, credentials.CREDS_FNAME
    credentials.CONFIG_DIR = 'jovian/tests/resources/creds/' + config_dir
    credentials.CREDS_FNAME = creds_filename
    creds = {
        "GUEST_KEY": "b6538d4dfde04fcf993463a828a9cec6",
        "API_URL": "https://api-staging.jovian.ai",
        "WEBAPP_URL": "https://staging.jovian.ml/",
        "ORG_ID": "staging",
        "API_KEY": "fake_api_key"
    }
    write_creds(creds)
    try:
        yield
    finally:
        purge_config()
    credentials.CONFIG_DIR = _d
    credentials.CREDS_FNAME = _f



from unittest import TestCase, mock

from jovian.utils.jupyter import get_notebook_server_path, has_ipynb_shell, in_notebook


@mock.patch("IPython.get_ipython")
def test_has_ipynb_shell_true(mock_get_ipython):
    mock_get_ipython().__class__.__name__ = 'ZMQInteractiveShell'
    assert has_ipynb_shell() == True


@mock.patch("IPython.get_ipython")
def test_has_ipynb_shell_terminal(mock_get_ipython):
    mock_get_ipython().__class__.__name__ = 'TerminalInteractiveShell'
    assert has_ipynb_shell() == False


@mock.patch('builtins.__import__', return_value=ImportError('fake import error'))
@mock.patch("IPython.get_ipython")
def test_has_ipynb_shell_except_return_false(mock_get_ipython, mock_import):
    assert has_ipynb_shell() == False


@mock.patch("ipykernel.kernelapp.IPKernelApp")
@mock.patch("IPython.get_ipython")
def test_in_notebook_false(mock_get_ipython, mock_IPKernelApp):
    mock_get_ipython().__class__.__name__ = 'ZMQInteractiveShell'
    mock_IPKernelApp.initialized.return_value = False
    assert in_notebook() == False

    mock_IPKernelApp.initialized.return_value = True
    assert in_notebook() == True



from unittest import TestCase, mock
from jovian._version import __version__
from pkg_resources import parse_version

from jovian.utils.latest import _get_latest_version, _print_update_message, check_update


class MockResponse:
    def __init__(self, json_data, status_code):
        self.json_data = json_data
        self.status_code = status_code

    def json(self):
        return self.json_data


@mock.patch("requests.get", mock.Mock(return_value=MockResponse({'info': {'version': '0.2.0.dev1'}}, 200)))
def test_get_latest_version_normal():
    expected_result = "0.2.0.dev1"
    assert _get_latest_version() == expected_result


@mock.patch("requests.get", mock.Mock(return_value=ConnectionError('fake requests.get error')))
def test_get_latest_version_exception():
    expected_result = __version__
    assert _get_latest_version() == expected_result


@mock.patch("jovian.utils.latest.in_notebook", mock.Mock(return_value=True))
def test_print_update_message_in_notebook(capsys):
    expected_result = """[jovian] Update Available: 0.0.1 --> 0.0.2
[jovian] Run `!pip install jovian --upgrade` to upgrade"""

    _print_update_message('0.0.1', '0.0.2')
    captured = capsys.readouterr()
    assert captured.out.strip() == expected_result


@mock.patch("jovian.utils.latest.in_notebook", mock.Mock(return_value=False))
def test_print_update_message_not_in_notebook(capsys):
    expected_result = """[jovian] Update Available: 0.0.1 --> 0.0.2
[jovian] Run `pip install jovian --upgrade` to upgrade"""

    _print_update_message('0.0.1', '0.0.2')
    captured = capsys.readouterr()
    assert captured.out.strip() == expected_result


@mock.patch("jovian.utils.latest._get_latest_version", mock.Mock(return_value='0.0.2'))
def test_check_update(capsys):
    expected_result = """[jovian] Update Available: 0.0.1 --> 0.0.2
[jovian] Run `pip install jovian --upgrade` to upgrade"""

    import jovian
    jovian.utils.latest.__version__ = '0.0.1'

    check_update(1)
    captured = capsys.readouterr()
    assert captured.out.strip() == expected_result


@mock.patch("jovian.utils.latest._get_latest_version", mock.Mock(return_value='0.0.2'))
@mock.patch("jovian.utils.latest.random", mock.Mock(return_value=0.5))
def test_check_update_mock_random(capsys):
    expected_result = """[jovian] Update Available: 0.0.1 --> 0.0.2
[jovian] Run `pip install jovian --upgrade` to upgrade"""

    import jovian
    jovian.utils.latest.__version__ = '0.0.1'

    check_update()
    captured = capsys.readouterr()
    assert captured.out.strip() == expected_result


@mock.patch("jovian.utils.latest._get_latest_version", mock.Mock(return_value='0.0.2'))
@mock.patch("jovian.utils.latest.random", mock.Mock(return_value=0.9))
def test_check_update_mock_random_no_output(capsys):
    expected_result = ""

    import jovian
    jovian.utils.latest.__version__ = '0.0.1'

    check_update()
    captured = capsys.readouterr()
    assert captured.out.strip() == expected_result
